@@ -19,7 +19,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 
 %%----------------------------------------------------------------------
@@ -37,7 +38,6 @@
 
 -behaviour(gen_server).
 
--include("ts_profile.hrl").
 -include("ts_config.hrl").
 
 %% External exports
@@ -106,12 +106,12 @@ add(nocache,Data) ->
 add(Data) ->
     ts_mon_cache:add(Data).
 
-add_match(Data,{UserId,SessionId,RequestId}) ->
-    add_match(Data,{UserId,SessionId,RequestId,[]});
-add_match(Data=[Head|_],{UserId,SessionId,RequestId,Bin}) ->
-    TimeStamp=now(),
+add_match(Data,{UserId,SessionId,RequestId,Tr,Name}) ->
+    add_match(Data,{UserId,SessionId,RequestId,[],Tr,Name});
+add_match(Data=[Head|_],{UserId,SessionId,RequestId,Bin,Tr,Name}) ->
+    TimeStamp=?NOW,
     put(last_match,Head),
-    ts_mon_cache:add_match(Data,{UserId,SessionId,RequestId,TimeStamp, Bin}).
+    ts_mon_cache:add_match(Data,{UserId,SessionId,RequestId,TimeStamp, Bin,Tr,Name}).
 
 status() ->
     gen_server:call({global, ?MODULE}, {status}).
@@ -131,17 +131,17 @@ endclient({Who, When, Elapsed}) ->
 sendmes({none, _, _})       -> skip;
 sendmes({protocol, _, _})   -> skip;
 sendmes({_Type, Who, What}) ->
-    gen_server:cast({global, ?MODULE}, {sendmsg, Who, now(), What}).
+    gen_server:cast({global, ?MODULE}, {sendmsg, Who, ?NOW, What}).
 
 rcvmes({none, _, _})    -> skip;
 rcvmes({protocol, _, _})-> skip;
 rcvmes({_, _, closed})  -> skip;
 rcvmes({_Type, Who, What})  ->
-    gen_server:cast({global, ?MODULE}, {rcvmsg, Who, now(), What}).
+    gen_server:cast({global, ?MODULE}, {rcvmsg, Who, ?NOW, What}).
 
 dump({none, _, _})-> skip;
 dump({_Type, Who, What})  ->
-    gen_server:cast({global, ?MODULE}, {dump, Who, now(), What}).
+    gen_server:cast({global, ?MODULE}, {dump, Who, ?NOW, What}).
 
 launcher_is_alive() ->
     gen_server:cast({global, ?MODULE}, {launcher_is_alive}).
@@ -159,12 +159,12 @@ launcher_is_alive() ->
 %%          {stop, Reason}
 %%----------------------------------------------------------------------
 init([LogDir]) ->
-    ?LOGF("Init, log dir is ~p~n",[LogDir],?NOTICE),
+    ?LOGF("Init, log dir is ~p~n",[LogDir],?INFO),
     Stats = #stats{os_mon  = dict:new()},
     State=#state{ dump_interval = ?config(dumpstats_interval),
                   log_dir   = LogDir,
                   stats     = Stats,
-                  lastdate  = now(),
+                  lastdate  = ?NOW,
                   laststats = Stats
                 },
     case ?config(mon_file) of
@@ -174,7 +174,7 @@ init([LogDir]) ->
             Filename = filename:join(LogDir, Name),
             case file:open(Filename,[write]) of
                 {ok, Stream} ->
-                    ?LOG("starting monitor~n",?NOTICE),
+                    ?LOG("starting monitor~n",?INFO),
                     {ok, State#state{log=Stream}};
                 {error, Reason} ->
                     ?LOGF("Can't open mon log file! ~p~n",[Reason], ?ERR),
@@ -197,7 +197,7 @@ handle_call({start_logger, Machines, DumpType, Backend}, From, State) ->
 %%% get status
 handle_call({status}, _From, State) ->
     Request   = ts_stats_mon:status(request),
-    Interval  = ts_utils:elapsed(State#state.lastdate, now()) / 1000,
+    Interval  = ts_utils:elapsed(State#state.lastdate, ?NOW) / 1000,
     Phase     = ts_stats_mon:status(newphase,sum),
     Connected =  case ts_stats_mon:status(connected,sum) of
                      {ok, Val} -> Val;
@@ -284,7 +284,7 @@ handle_cast({dumpstats}, State=#state{stats=Stats}) ->
     NewSessions = ts_stats_mon:reset_all_stats(Stats#stats.session),
     NewOSmon = ts_stats_mon:reset_all_stats(Stats#stats.os_mon),
     NewStats = Stats#stats{session=NewSessions, os_mon=NewOSmon},
-    {noreply, State#state{laststats = Stats, stats=NewStats,lastdate=now()}};
+    {noreply, State#state{laststats = Stats, stats=NewStats,lastdate=?NOW}};
 
 
 handle_cast({sendmsg, _, _, _}, State = #state{type = none}) ->
@@ -332,7 +332,7 @@ handle_cast({stop}, State=#state{launchers=L}) -> % we should stop, wait until n
     {noreply, State#state{stop = true, launchers=L-1}};
 
 handle_cast({launcher_is_alive}, State=#state{launchers=L}) ->
-    ?LOG("A launcher has started~n", ?NOTICE),
+    ?LOG("A launcher has started~n", ?INFO),
     {noreply, State#state{launchers=L+1}};
 
 
@@ -369,7 +369,7 @@ terminate(Reason, State) ->
         json ->
             io:format(State#state.log,"]}]}~n",[]);
         _ ->
-            io:format(State#state.log,"EndMonitor:~w~n",[now()])
+            io:format(State#state.log,"EndMonitor:~w~n",[?NOW])
         end,
     case State#state.log of
         standard_io -> ok;
@@ -437,7 +437,7 @@ start_dump(State=#state{type=Type}) ->
             ?LOG("dump file opened, starting monitor~n",?INFO),
             case Type of
                 protocol ->
-                    io:format(Stream,"#date;pid;id;http method;host;URL;HTTP status;size;match;error~n",[]);
+                    io:format(Stream,"#date;pid;id;http method;host;URL;HTTP status;size;duration;transaction;match;error;tag~n",[]);
                 _ ->
                     ok
             end,
@@ -485,10 +485,10 @@ export_stats_common(BackEnd, Stats,LastStats,Log)->
 %% @doc start the launcher on clients nodes
 %%----------------------------------------------------------------------
 start_launchers(Machines) ->
-    ?DebugF("Need to start tsung client on ~p~n",[Machines]),
+    ?LOGF("Tsung clients setup: ~p~n",[Machines],?DEB),
     GetHost = fun(A) -> list_to_atom(A#client.host) end,
     HostList = lists:map(GetHost, Machines),
-    ?DebugF("Hostlist is ~p~n",[HostList]),
+    ?LOGF("Starting tsung clients on hosts: ~p~n",[HostList],?NOTICE),
     %% starts beam on all client hosts
     ts_config_server:newbeams(HostList).
 
